# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

import datetime
from typing import Dict, List, Optional, Union

import msrest.serialization

from ._container_registry_management_client_enums import *


class AgentProperties(msrest.serialization.Model):
    """The properties that determine the run agent configuration.

    :param cpu: The CPU configuration in terms of number of cores required for the run.
    :type cpu: int
    """

    _attribute_map = {
        'cpu': {'key': 'cpu', 'type': 'int'},
    }

    def __init__(
        self,
        *,
        cpu: Optional[int] = None,
        **kwargs
    ):
        super(AgentProperties, self).__init__(**kwargs)
        self.cpu = cpu


class Argument(msrest.serialization.Model):
    """The properties of a run argument.

    All required parameters must be populated in order to send to Azure.

    :param name: Required. The name of the argument.
    :type name: str
    :param value: Required. The value of the argument.
    :type value: str
    :param is_secret: Flag to indicate whether the argument represents a secret and want to be
     removed from build logs.
    :type is_secret: bool
    """

    _validation = {
        'name': {'required': True},
        'value': {'required': True},
    }

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'value': {'key': 'value', 'type': 'str'},
        'is_secret': {'key': 'isSecret', 'type': 'bool'},
    }

    def __init__(
        self,
        *,
        name: str,
        value: str,
        is_secret: Optional[bool] = False,
        **kwargs
    ):
        super(Argument, self).__init__(**kwargs)
        self.name = name
        self.value = value
        self.is_secret = is_secret


class AuthInfo(msrest.serialization.Model):
    """The authorization properties for accessing the source code repository.

    All required parameters must be populated in order to send to Azure.

    :param token_type: Required. The type of Auth token. Possible values include: "PAT", "OAuth".
    :type token_type: str or ~azure.mgmt.containerregistry.v2019_04_01.models.TokenType
    :param token: Required. The access token used to access the source control provider.
    :type token: str
    :param refresh_token: The refresh token used to refresh the access token.
    :type refresh_token: str
    :param scope: The scope of the access token.
    :type scope: str
    :param expires_in: Time in seconds that the token remains valid.
    :type expires_in: int
    """

    _validation = {
        'token_type': {'required': True},
        'token': {'required': True},
    }

    _attribute_map = {
        'token_type': {'key': 'tokenType', 'type': 'str'},
        'token': {'key': 'token', 'type': 'str'},
        'refresh_token': {'key': 'refreshToken', 'type': 'str'},
        'scope': {'key': 'scope', 'type': 'str'},
        'expires_in': {'key': 'expiresIn', 'type': 'int'},
    }

    def __init__(
        self,
        *,
        token_type: Union[str, "TokenType"],
        token: str,
        refresh_token: Optional[str] = None,
        scope: Optional[str] = None,
        expires_in: Optional[int] = None,
        **kwargs
    ):
        super(AuthInfo, self).__init__(**kwargs)
        self.token_type = token_type
        self.token = token
        self.refresh_token = refresh_token
        self.scope = scope
        self.expires_in = expires_in


class AuthInfoUpdateParameters(msrest.serialization.Model):
    """The authorization properties for accessing the source code repository.

    :param token_type: The type of Auth token. Possible values include: "PAT", "OAuth".
    :type token_type: str or ~azure.mgmt.containerregistry.v2019_04_01.models.TokenType
    :param token: The access token used to access the source control provider.
    :type token: str
    :param refresh_token: The refresh token used to refresh the access token.
    :type refresh_token: str
    :param scope: The scope of the access token.
    :type scope: str
    :param expires_in: Time in seconds that the token remains valid.
    :type expires_in: int
    """

    _attribute_map = {
        'token_type': {'key': 'tokenType', 'type': 'str'},
        'token': {'key': 'token', 'type': 'str'},
        'refresh_token': {'key': 'refreshToken', 'type': 'str'},
        'scope': {'key': 'scope', 'type': 'str'},
        'expires_in': {'key': 'expiresIn', 'type': 'int'},
    }

    def __init__(
        self,
        *,
        token_type: Optional[Union[str, "TokenType"]] = None,
        token: Optional[str] = None,
        refresh_token: Optional[str] = None,
        scope: Optional[str] = None,
        expires_in: Optional[int] = None,
        **kwargs
    ):
        super(AuthInfoUpdateParameters, self).__init__(**kwargs)
        self.token_type = token_type
        self.token = token
        self.refresh_token = refresh_token
        self.scope = scope
        self.expires_in = expires_in


class BaseImageDependency(msrest.serialization.Model):
    """Properties that describe a base image dependency.

    :param type: The type of the base image dependency. Possible values include: "BuildTime",
     "RunTime".
    :type type: str or ~azure.mgmt.containerregistry.v2019_04_01.models.BaseImageDependencyType
    :param registry: The registry login server.
    :type registry: str
    :param repository: The repository name.
    :type repository: str
    :param tag: The tag name.
    :type tag: str
    :param digest: The sha256-based digest of the image manifest.
    :type digest: str
    """

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'registry': {'key': 'registry', 'type': 'str'},
        'repository': {'key': 'repository', 'type': 'str'},
        'tag': {'key': 'tag', 'type': 'str'},
        'digest': {'key': 'digest', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        type: Optional[Union[str, "BaseImageDependencyType"]] = None,
        registry: Optional[str] = None,
        repository: Optional[str] = None,
        tag: Optional[str] = None,
        digest: Optional[str] = None,
        **kwargs
    ):
        super(BaseImageDependency, self).__init__(**kwargs)
        self.type = type
        self.registry = registry
        self.repository = repository
        self.tag = tag
        self.digest = digest


class BaseImageTrigger(msrest.serialization.Model):
    """The trigger based on base image dependency.

    All required parameters must be populated in order to send to Azure.

    :param base_image_trigger_type: Required. The type of the auto trigger for base image
     dependency updates. Possible values include: "All", "Runtime".
    :type base_image_trigger_type: str or
     ~azure.mgmt.containerregistry.v2019_04_01.models.BaseImageTriggerType
    :param status: The current status of trigger. Possible values include: "Disabled", "Enabled".
    :type status: str or ~azure.mgmt.containerregistry.v2019_04_01.models.TriggerStatus
    :param name: Required. The name of the trigger.
    :type name: str
    """

    _validation = {
        'base_image_trigger_type': {'required': True},
        'name': {'required': True},
    }

    _attribute_map = {
        'base_image_trigger_type': {'key': 'baseImageTriggerType', 'type': 'str'},
        'status': {'key': 'status', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        base_image_trigger_type: Union[str, "BaseImageTriggerType"],
        name: str,
        status: Optional[Union[str, "TriggerStatus"]] = None,
        **kwargs
    ):
        super(BaseImageTrigger, self).__init__(**kwargs)
        self.base_image_trigger_type = base_image_trigger_type
        self.status = status
        self.name = name


class BaseImageTriggerUpdateParameters(msrest.serialization.Model):
    """The properties for updating base image dependency trigger.

    All required parameters must be populated in order to send to Azure.

    :param base_image_trigger_type: The type of the auto trigger for base image dependency updates.
     Possible values include: "All", "Runtime".
    :type base_image_trigger_type: str or
     ~azure.mgmt.containerregistry.v2019_04_01.models.BaseImageTriggerType
    :param status: The current status of trigger. Possible values include: "Disabled", "Enabled".
    :type status: str or ~azure.mgmt.containerregistry.v2019_04_01.models.TriggerStatus
    :param name: Required. The name of the trigger.
    :type name: str
    """

    _validation = {
        'name': {'required': True},
    }

    _attribute_map = {
        'base_image_trigger_type': {'key': 'baseImageTriggerType', 'type': 'str'},
        'status': {'key': 'status', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        name: str,
        base_image_trigger_type: Optional[Union[str, "BaseImageTriggerType"]] = None,
        status: Optional[Union[str, "TriggerStatus"]] = None,
        **kwargs
    ):
        super(BaseImageTriggerUpdateParameters, self).__init__(**kwargs)
        self.base_image_trigger_type = base_image_trigger_type
        self.status = status
        self.name = name


class Credentials(msrest.serialization.Model):
    """The parameters that describes a set of credentials that will be used when a run is invoked.

    :param source_registry: Describes the credential parameters for accessing the source registry.
    :type source_registry:
     ~azure.mgmt.containerregistry.v2019_04_01.models.SourceRegistryCredentials
    :param custom_registries: Describes the credential parameters for accessing other custom
     registries. The key
     for the dictionary item will be the registry login server (myregistry.azurecr.io) and
     the value of the item will be the registry credentials for accessing the registry.
    :type custom_registries: dict[str,
     ~azure.mgmt.containerregistry.v2019_04_01.models.CustomRegistryCredentials]
    """

    _attribute_map = {
        'source_registry': {'key': 'sourceRegistry', 'type': 'SourceRegistryCredentials'},
        'custom_registries': {'key': 'customRegistries', 'type': '{CustomRegistryCredentials}'},
    }

    def __init__(
        self,
        *,
        source_registry: Optional["SourceRegistryCredentials"] = None,
        custom_registries: Optional[Dict[str, "CustomRegistryCredentials"]] = None,
        **kwargs
    ):
        super(Credentials, self).__init__(**kwargs)
        self.source_registry = source_registry
        self.custom_registries = custom_registries


class CustomRegistryCredentials(msrest.serialization.Model):
    """Describes the credentials that will be used to access a custom registry during a run.

    :param user_name: The username for logging into the custom registry.
    :type user_name: ~azure.mgmt.containerregistry.v2019_04_01.models.SecretObject
    :param password: The password for logging into the custom registry. The password is a secret
     object that allows multiple ways of providing the value for it.
    :type password: ~azure.mgmt.containerregistry.v2019_04_01.models.SecretObject
    :param identity: Indicates the managed identity assigned to the custom credential. If a
     user-assigned identity
     this value is the Client ID. If a system-assigned identity, the value will be ``system``. In
     the case of a system-assigned identity, the Client ID will be determined by the runner. This
     identity may be used to authenticate to key vault to retrieve credentials or it may be the
     only
     source of authentication used for accessing the registry.
    :type identity: str
    """

    _attribute_map = {
        'user_name': {'key': 'userName', 'type': 'SecretObject'},
        'password': {'key': 'password', 'type': 'SecretObject'},
        'identity': {'key': 'identity', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        user_name: Optional["SecretObject"] = None,
        password: Optional["SecretObject"] = None,
        identity: Optional[str] = None,
        **kwargs
    ):
        super(CustomRegistryCredentials, self).__init__(**kwargs)
        self.user_name = user_name
        self.password = password
        self.identity = identity


class RunRequest(msrest.serialization.Model):
    """The request parameters for scheduling a run.

    You probably want to use the sub-classes and not this class directly. Known
    sub-classes are: DockerBuildRequest, EncodedTaskRunRequest, FileTaskRunRequest, TaskRunRequest.

    All required parameters must be populated in order to send to Azure.

    :param type: Required. The type of the run request.Constant filled by server.
    :type type: str
    :param is_archive_enabled: The value that indicates whether archiving is enabled for the run or
     not.
    :type is_archive_enabled: bool
    """

    _validation = {
        'type': {'required': True},
    }

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'is_archive_enabled': {'key': 'isArchiveEnabled', 'type': 'bool'},
    }

    _subtype_map = {
        'type': {'DockerBuildRequest': 'DockerBuildRequest', 'EncodedTaskRunRequest': 'EncodedTaskRunRequest', 'FileTaskRunRequest': 'FileTaskRunRequest', 'TaskRunRequest': 'TaskRunRequest'}
    }

    def __init__(
        self,
        *,
        is_archive_enabled: Optional[bool] = False,
        **kwargs
    ):
        super(RunRequest, self).__init__(**kwargs)
        self.type = None  # type: Optional[str]
        self.is_archive_enabled = is_archive_enabled


class DockerBuildRequest(RunRequest):
    """The parameters for a docker quick build.

    All required parameters must be populated in order to send to Azure.

    :param type: Required. The type of the run request.Constant filled by server.
    :type type: str
    :param is_archive_enabled: The value that indicates whether archiving is enabled for the run or
     not.
    :type is_archive_enabled: bool
    :param image_names: The fully qualified image names including the repository and tag.
    :type image_names: list[str]
    :param is_push_enabled: The value of this property indicates whether the image built should be
     pushed to the registry or not.
    :type is_push_enabled: bool
    :param no_cache: The value of this property indicates whether the image cache is enabled or
     not.
    :type no_cache: bool
    :param docker_file_path: Required. The Docker file path relative to the source location.
    :type docker_file_path: str
    :param target: The name of the target build stage for the docker build.
    :type target: str
    :param arguments: The collection of override arguments to be used when executing the run.
    :type arguments: list[~azure.mgmt.containerregistry.v2019_04_01.models.Argument]
    :param timeout: Run timeout in seconds.
    :type timeout: int
    :param platform: Required. The platform properties against which the run has to happen.
    :type platform: ~azure.mgmt.containerregistry.v2019_04_01.models.PlatformProperties
    :param agent_configuration: The machine configuration of the run agent.
    :type agent_configuration: ~azure.mgmt.containerregistry.v2019_04_01.models.AgentProperties
    :param source_location: The URL(absolute or relative) of the source context. It can be an URL
     to a tar or git repository.
     If it is relative URL, the relative path should be obtained from calling
     listBuildSourceUploadUrl API.
    :type source_location: str
    :param credentials: The properties that describes a set of credentials that will be used when
     this run is invoked.
    :type credentials: ~azure.mgmt.containerregistry.v2019_04_01.models.Credentials
    """

    _validation = {
        'type': {'required': True},
        'docker_file_path': {'required': True},
        'timeout': {'maximum': 28800, 'minimum': 300},
        'platform': {'required': True},
    }

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'is_archive_enabled': {'key': 'isArchiveEnabled', 'type': 'bool'},
        'image_names': {'key': 'imageNames', 'type': '[str]'},
        'is_push_enabled': {'key': 'isPushEnabled', 'type': 'bool'},
        'no_cache': {'key': 'noCache', 'type': 'bool'},
        'docker_file_path': {'key': 'dockerFilePath', 'type': 'str'},
        'target': {'key': 'target', 'type': 'str'},
        'arguments': {'key': 'arguments', 'type': '[Argument]'},
        'timeout': {'key': 'timeout', 'type': 'int'},
        'platform': {'key': 'platform', 'type': 'PlatformProperties'},
        'agent_configuration': {'key': 'agentConfiguration', 'type': 'AgentProperties'},
        'source_location': {'key': 'sourceLocation', 'type': 'str'},
        'credentials': {'key': 'credentials', 'type': 'Credentials'},
    }

    def __init__(
        self,
        *,
        docker_file_path: str,
        platform: "PlatformProperties",
        is_archive_enabled: Optional[bool] = False,
        image_names: Optional[List[str]] = None,
        is_push_enabled: Optional[bool] = True,
        no_cache: Optional[bool] = False,
        target: Optional[str] = None,
        arguments: Optional[List["Argument"]] = None,
        timeout: Optional[int] = 3600,
        agent_configuration: Optional["AgentProperties"] = None,
        source_location: Optional[str] = None,
        credentials: Optional["Credentials"] = None,
        **kwargs
    ):
        super(DockerBuildRequest, self).__init__(is_archive_enabled=is_archive_enabled, **kwargs)
        self.type = 'DockerBuildRequest'  # type: str
        self.image_names = image_names
        self.is_push_enabled = is_push_enabled
        self.no_cache = no_cache
        self.docker_file_path = docker_file_path
        self.target = target
        self.arguments = arguments
        self.timeout = timeout
        self.platform = platform
        self.agent_configuration = agent_configuration
        self.source_location = source_location
        self.credentials = credentials


class TaskStepProperties(msrest.serialization.Model):
    """Base properties for any task step.

    You probably want to use the sub-classes and not this class directly. Known
    sub-classes are: DockerBuildStep, EncodedTaskStep, FileTaskStep.

    Variables are only populated by the server, and will be ignored when sending a request.

    All required parameters must be populated in order to send to Azure.

    :param type: Required. The type of the step.Constant filled by server.  Possible values
     include: "Docker", "FileTask", "EncodedTask".
    :type type: str or ~azure.mgmt.containerregistry.v2019_04_01.models.StepType
    :ivar base_image_dependencies: List of base image dependencies for a step.
    :vartype base_image_dependencies:
     list[~azure.mgmt.containerregistry.v2019_04_01.models.BaseImageDependency]
    :param context_path: The URL(absolute or relative) of the source context for the task step.
    :type context_path: str
    :param context_access_token: The token (git PAT or SAS token of storage account blob)
     associated with the context for a step.
    :type context_access_token: str
    """

    _validation = {
        'type': {'required': True},
        'base_image_dependencies': {'readonly': True},
    }

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'base_image_dependencies': {'key': 'baseImageDependencies', 'type': '[BaseImageDependency]'},
        'context_path': {'key': 'contextPath', 'type': 'str'},
        'context_access_token': {'key': 'contextAccessToken', 'type': 'str'},
    }

    _subtype_map = {
        'type': {'Docker': 'DockerBuildStep', 'EncodedTask': 'EncodedTaskStep', 'FileTask': 'FileTaskStep'}
    }

    def __init__(
        self,
        *,
        context_path: Optional[str] = None,
        context_access_token: Optional[str] = None,
        **kwargs
    ):
        super(TaskStepProperties, self).__init__(**kwargs)
        self.type = None  # type: Optional[str]
        self.base_image_dependencies = None
        self.context_path = context_path
        self.context_access_token = context_access_token


class DockerBuildStep(TaskStepProperties):
    """The Docker build step.

    Variables are only populated by the server, and will be ignored when sending a request.

    All required parameters must be populated in order to send to Azure.

    :param type: Required. The type of the step.Constant filled by server.  Possible values
     include: "Docker", "FileTask", "EncodedTask".
    :type type: str or ~azure.mgmt.containerregistry.v2019_04_01.models.StepType
    :ivar base_image_dependencies: List of base image dependencies for a step.
    :vartype base_image_dependencies:
     list[~azure.mgmt.containerregistry.v2019_04_01.models.BaseImageDependency]
    :param context_path: The URL(absolute or relative) of the source context for the task step.
    :type context_path: str
    :param context_access_token: The token (git PAT or SAS token of storage account blob)
     associated with the context for a step.
    :type context_access_token: str
    :param image_names: The fully qualified image names including the repository and tag.
    :type image_names: list[str]
    :param is_push_enabled: The value of this property indicates whether the image built should be
     pushed to the registry or not.
    :type is_push_enabled: bool
    :param no_cache: The value of this property indicates whether the image cache is enabled or
     not.
    :type no_cache: bool
    :param docker_file_path: Required. The Docker file path relative to the source context.
    :type docker_file_path: str
    :param target: The name of the target build stage for the docker build.
    :type target: str
    :param arguments: The collection of override arguments to be used when executing this build
     step.
    :type arguments: list[~azure.mgmt.containerregistry.v2019_04_01.models.Argument]
    """

    _validation = {
        'type': {'required': True},
        'base_image_dependencies': {'readonly': True},
        'docker_file_path': {'required': True},
    }

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'base_image_dependencies': {'key': 'baseImageDependencies', 'type': '[BaseImageDependency]'},
        'context_path': {'key': 'contextPath', 'type': 'str'},
        'context_access_token': {'key': 'contextAccessToken', 'type': 'str'},
        'image_names': {'key': 'imageNames', 'type': '[str]'},
        'is_push_enabled': {'key': 'isPushEnabled', 'type': 'bool'},
        'no_cache': {'key': 'noCache', 'type': 'bool'},
        'docker_file_path': {'key': 'dockerFilePath', 'type': 'str'},
        'target': {'key': 'target', 'type': 'str'},
        'arguments': {'key': 'arguments', 'type': '[Argument]'},
    }

    def __init__(
        self,
        *,
        docker_file_path: str,
        context_path: Optional[str] = None,
        context_access_token: Optional[str] = None,
        image_names: Optional[List[str]] = None,
        is_push_enabled: Optional[bool] = True,
        no_cache: Optional[bool] = False,
        target: Optional[str] = None,
        arguments: Optional[List["Argument"]] = None,
        **kwargs
    ):
        super(DockerBuildStep, self).__init__(context_path=context_path, context_access_token=context_access_token, **kwargs)
        self.type = 'Docker'  # type: str
        self.image_names = image_names
        self.is_push_enabled = is_push_enabled
        self.no_cache = no_cache
        self.docker_file_path = docker_file_path
        self.target = target
        self.arguments = arguments


class TaskStepUpdateParameters(msrest.serialization.Model):
    """Base properties for updating any task step.

    You probably want to use the sub-classes and not this class directly. Known
    sub-classes are: DockerBuildStepUpdateParameters, EncodedTaskStepUpdateParameters, FileTaskStepUpdateParameters.

    All required parameters must be populated in order to send to Azure.

    :param type: Required. The type of the step.Constant filled by server.  Possible values
     include: "Docker", "FileTask", "EncodedTask".
    :type type: str or ~azure.mgmt.containerregistry.v2019_04_01.models.StepType
    :param context_path: The URL(absolute or relative) of the source context for the task step.
    :type context_path: str
    :param context_access_token: The token (git PAT or SAS token of storage account blob)
     associated with the context for a step.
    :type context_access_token: str
    """

    _validation = {
        'type': {'required': True},
    }

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'context_path': {'key': 'contextPath', 'type': 'str'},
        'context_access_token': {'key': 'contextAccessToken', 'type': 'str'},
    }

    _subtype_map = {
        'type': {'Docker': 'DockerBuildStepUpdateParameters', 'EncodedTask': 'EncodedTaskStepUpdateParameters', 'FileTask': 'FileTaskStepUpdateParameters'}
    }

    def __init__(
        self,
        *,
        context_path: Optional[str] = None,
        context_access_token: Optional[str] = None,
        **kwargs
    ):
        super(TaskStepUpdateParameters, self).__init__(**kwargs)
        self.type = None  # type: Optional[str]
        self.context_path = context_path
        self.context_access_token = context_access_token


class DockerBuildStepUpdateParameters(TaskStepUpdateParameters):
    """The properties for updating a docker build step.

    All required parameters must be populated in order to send to Azure.

    :param type: Required. The type of the step.Constant filled by server.  Possible values
     include: "Docker", "FileTask", "EncodedTask".
    :type type: str or ~azure.mgmt.containerregistry.v2019_04_01.models.StepType
    :param context_path: The URL(absolute or relative) of the source context for the task step.
    :type context_path: str
    :param context_access_token: The token (git PAT or SAS token of storage account blob)
     associated with the context for a step.
    :type context_access_token: str
    :param image_names: The fully qualified image names including the repository and tag.
    :type image_names: list[str]
    :param is_push_enabled: The value of this property indicates whether the image built should be
     pushed to the registry or not.
    :type is_push_enabled: bool
    :param no_cache: The value of this property indicates whether the image cache is enabled or
     not.
    :type no_cache: bool
    :param docker_file_path: The Docker file path relative to the source context.
    :type docker_file_path: str
    :param arguments: The collection of override arguments to be used when executing this build
     step.
    :type arguments: list[~azure.mgmt.containerregistry.v2019_04_01.models.Argument]
    :param target: The name of the target build stage for the docker build.
    :type target: str
    """

    _validation = {
        'type': {'required': True},
    }

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'context_path': {'key': 'contextPath', 'type': 'str'},
        'context_access_token': {'key': 'contextAccessToken', 'type': 'str'},
        'image_names': {'key': 'imageNames', 'type': '[str]'},
        'is_push_enabled': {'key': 'isPushEnabled', 'type': 'bool'},
        'no_cache': {'key': 'noCache', 'type': 'bool'},
        'docker_file_path': {'key': 'dockerFilePath', 'type': 'str'},
        'arguments': {'key': 'arguments', 'type': '[Argument]'},
        'target': {'key': 'target', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        context_path: Optional[str] = None,
        context_access_token: Optional[str] = None,
        image_names: Optional[List[str]] = None,
        is_push_enabled: Optional[bool] = None,
        no_cache: Optional[bool] = None,
        docker_file_path: Optional[str] = None,
        arguments: Optional[List["Argument"]] = None,
        target: Optional[str] = None,
        **kwargs
    ):
        super(DockerBuildStepUpdateParameters, self).__init__(context_path=context_path, context_access_token=context_access_token, **kwargs)
        self.type = 'Docker'  # type: str
        self.image_names = image_names
        self.is_push_enabled = is_push_enabled
        self.no_cache = no_cache
        self.docker_file_path = docker_file_path
        self.arguments = arguments
        self.target = target


class EncodedTaskRunRequest(RunRequest):
    """The parameters for a quick task run request.

    All required parameters must be populated in order to send to Azure.

    :param type: Required. The type of the run request.Constant filled by server.
    :type type: str
    :param is_archive_enabled: The value that indicates whether archiving is enabled for the run or
     not.
    :type is_archive_enabled: bool
    :param encoded_task_content: Required. Base64 encoded value of the template/definition file
     content.
    :type encoded_task_content: str
    :param encoded_values_content: Base64 encoded value of the parameters/values file content.
    :type encoded_values_content: str
    :param values: The collection of overridable values that can be passed when running a task.
    :type values: list[~azure.mgmt.containerregistry.v2019_04_01.models.SetValue]
    :param timeout: Run timeout in seconds.
    :type timeout: int
    :param platform: Required. The platform properties against which the run has to happen.
    :type platform: ~azure.mgmt.containerregistry.v2019_04_01.models.PlatformProperties
    :param agent_configuration: The machine configuration of the run agent.
    :type agent_configuration: ~azure.mgmt.containerregistry.v2019_04_01.models.AgentProperties
    :param source_location: The URL(absolute or relative) of the source context. It can be an URL
     to a tar or git repository.
     If it is relative URL, the relative path should be obtained from calling
     listBuildSourceUploadUrl API.
    :type source_location: str
    :param credentials: The properties that describes a set of credentials that will be used when
     this run is invoked.
    :type credentials: ~azure.mgmt.containerregistry.v2019_04_01.models.Credentials
    """

    _validation = {
        'type': {'required': True},
        'encoded_task_content': {'required': True},
        'timeout': {'maximum': 28800, 'minimum': 300},
        'platform': {'required': True},
    }

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'is_archive_enabled': {'key': 'isArchiveEnabled', 'type': 'bool'},
        'encoded_task_content': {'key': 'encodedTaskContent', 'type': 'str'},
        'encoded_values_content': {'key': 'encodedValuesContent', 'type': 'str'},
        'values': {'key': 'values', 'type': '[SetValue]'},
        'timeout': {'key': 'timeout', 'type': 'int'},
        'platform': {'key': 'platform', 'type': 'PlatformProperties'},
        'agent_configuration': {'key': 'agentConfiguration', 'type': 'AgentProperties'},
        'source_location': {'key': 'sourceLocation', 'type': 'str'},
        'credentials': {'key': 'credentials', 'type': 'Credentials'},
    }

    def __init__(
        self,
        *,
        encoded_task_content: str,
        platform: "PlatformProperties",
        is_archive_enabled: Optional[bool] = False,
        encoded_values_content: Optional[str] = None,
        values: Optional[List["SetValue"]] = None,
        timeout: Optional[int] = 3600,
        agent_configuration: Optional["AgentProperties"] = None,
        source_location: Optional[str] = None,
        credentials: Optional["Credentials"] = None,
        **kwargs
    ):
        super(EncodedTaskRunRequest, self).__init__(is_archive_enabled=is_archive_enabled, **kwargs)
        self.type = 'EncodedTaskRunRequest'  # type: str
        self.encoded_task_content = encoded_task_content
        self.encoded_values_content = encoded_values_content
        self.values = values
        self.timeout = timeout
        self.platform = platform
        self.agent_configuration = agent_configuration
        self.source_location = source_location
        self.credentials = credentials


class EncodedTaskStep(TaskStepProperties):
    """The properties of a encoded task step.

    Variables are only populated by the server, and will be ignored when sending a request.

    All required parameters must be populated in order to send to Azure.

    :param type: Required. The type of the step.Constant filled by server.  Possible values
     include: "Docker", "FileTask", "EncodedTask".
    :type type: str or ~azure.mgmt.containerregistry.v2019_04_01.models.StepType
    :ivar base_image_dependencies: List of base image dependencies for a step.
    :vartype base_image_dependencies:
     list[~azure.mgmt.containerregistry.v2019_04_01.models.BaseImageDependency]
    :param context_path: The URL(absolute or relative) of the source context for the task step.
    :type context_path: str
    :param context_access_token: The token (git PAT or SAS token of storage account blob)
     associated with the context for a step.
    :type context_access_token: str
    :param encoded_task_content: Required. Base64 encoded value of the template/definition file
     content.
    :type encoded_task_content: str
    :param encoded_values_content: Base64 encoded value of the parameters/values file content.
    :type encoded_values_content: str
    :param values: The collection of overridable values that can be passed when running a task.
    :type values: list[~azure.mgmt.containerregistry.v2019_04_01.models.SetValue]
    """

    _validation = {
        'type': {'required': True},
        'base_image_dependencies': {'readonly': True},
        'encoded_task_content': {'required': True},
    }

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'base_image_dependencies': {'key': 'baseImageDependencies', 'type': '[BaseImageDependency]'},
        'context_path': {'key': 'contextPath', 'type': 'str'},
        'context_access_token': {'key': 'contextAccessToken', 'type': 'str'},
        'encoded_task_content': {'key': 'encodedTaskContent', 'type': 'str'},
        'encoded_values_content': {'key': 'encodedValuesContent', 'type': 'str'},
        'values': {'key': 'values', 'type': '[SetValue]'},
    }

    def __init__(
        self,
        *,
        encoded_task_content: str,
        context_path: Optional[str] = None,
        context_access_token: Optional[str] = None,
        encoded_values_content: Optional[str] = None,
        values: Optional[List["SetValue"]] = None,
        **kwargs
    ):
        super(EncodedTaskStep, self).__init__(context_path=context_path, context_access_token=context_access_token, **kwargs)
        self.type = 'EncodedTask'  # type: str
        self.encoded_task_content = encoded_task_content
        self.encoded_values_content = encoded_values_content
        self.values = values


class EncodedTaskStepUpdateParameters(TaskStepUpdateParameters):
    """The properties for updating encoded task step.

    All required parameters must be populated in order to send to Azure.

    :param type: Required. The type of the step.Constant filled by server.  Possible values
     include: "Docker", "FileTask", "EncodedTask".
    :type type: str or ~azure.mgmt.containerregistry.v2019_04_01.models.StepType
    :param context_path: The URL(absolute or relative) of the source context for the task step.
    :type context_path: str
    :param context_access_token: The token (git PAT or SAS token of storage account blob)
     associated with the context for a step.
    :type context_access_token: str
    :param encoded_task_content: Base64 encoded value of the template/definition file content.
    :type encoded_task_content: str
    :param encoded_values_content: Base64 encoded value of the parameters/values file content.
    :type encoded_values_content: str
    :param values: The collection of overridable values that can be passed when running a task.
    :type values: list[~azure.mgmt.containerregistry.v2019_04_01.models.SetValue]
    """

    _validation = {
        'type': {'required': True},
    }

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'context_path': {'key': 'contextPath', 'type': 'str'},
        'context_access_token': {'key': 'contextAccessToken', 'type': 'str'},
        'encoded_task_content': {'key': 'encodedTaskContent', 'type': 'str'},
        'encoded_values_content': {'key': 'encodedValuesContent', 'type': 'str'},
        'values': {'key': 'values', 'type': '[SetValue]'},
    }

    def __init__(
        self,
        *,
        context_path: Optional[str] = None,
        context_access_token: Optional[str] = None,
        encoded_task_content: Optional[str] = None,
        encoded_values_content: Optional[str] = None,
        values: Optional[List["SetValue"]] = None,
        **kwargs
    ):
        super(EncodedTaskStepUpdateParameters, self).__init__(context_path=context_path, context_access_token=context_access_token, **kwargs)
        self.type = 'EncodedTask'  # type: str
        self.encoded_task_content = encoded_task_content
        self.encoded_values_content = encoded_values_content
        self.values = values


class FileTaskRunRequest(RunRequest):
    """The request parameters for a scheduling run against a task file.

    All required parameters must be populated in order to send to Azure.

    :param type: Required. The type of the run request.Constant filled by server.
    :type type: str
    :param is_archive_enabled: The value that indicates whether archiving is enabled for the run or
     not.
    :type is_archive_enabled: bool
    :param task_file_path: Required. The template/definition file path relative to the source.
    :type task_file_path: str
    :param values_file_path: The values/parameters file path relative to the source.
    :type values_file_path: str
    :param values: The collection of overridable values that can be passed when running a task.
    :type values: list[~azure.mgmt.containerregistry.v2019_04_01.models.SetValue]
    :param timeout: Run timeout in seconds.
    :type timeout: int
    :param platform: Required. The platform properties against which the run has to happen.
    :type platform: ~azure.mgmt.containerregistry.v2019_04_01.models.PlatformProperties
    :param agent_configuration: The machine configuration of the run agent.
    :type agent_configuration: ~azure.mgmt.containerregistry.v2019_04_01.models.AgentProperties
    :param source_location: The URL(absolute or relative) of the source context. It can be an URL
     to a tar or git repository.
     If it is relative URL, the relative path should be obtained from calling
     listBuildSourceUploadUrl API.
    :type source_location: str
    :param credentials: The properties that describes a set of credentials that will be used when
     this run is invoked.
    :type credentials: ~azure.mgmt.containerregistry.v2019_04_01.models.Credentials
    """

    _validation = {
        'type': {'required': True},
        'task_file_path': {'required': True},
        'timeout': {'maximum': 28800, 'minimum': 300},
        'platform': {'required': True},
    }

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'is_archive_enabled': {'key': 'isArchiveEnabled', 'type': 'bool'},
        'task_file_path': {'key': 'taskFilePath', 'type': 'str'},
        'values_file_path': {'key': 'valuesFilePath', 'type': 'str'},
        'values': {'key': 'values', 'type': '[SetValue]'},
        'timeout': {'key': 'timeout', 'type': 'int'},
        'platform': {'key': 'platform', 'type': 'PlatformProperties'},
        'agent_configuration': {'key': 'agentConfiguration', 'type': 'AgentProperties'},
        'source_location': {'key': 'sourceLocation', 'type': 'str'},
        'credentials': {'key': 'credentials', 'type': 'Credentials'},
    }

    def __init__(
        self,
        *,
        task_file_path: str,
        platform: "PlatformProperties",
        is_archive_enabled: Optional[bool] = False,
        values_file_path: Optional[str] = None,
        values: Optional[List["SetValue"]] = None,
        timeout: Optional[int] = 3600,
        agent_configuration: Optional["AgentProperties"] = None,
        source_location: Optional[str] = None,
        credentials: Optional["Credentials"] = None,
        **kwargs
    ):
        super(FileTaskRunRequest, self).__init__(is_archive_enabled=is_archive_enabled, **kwargs)
        self.type = 'FileTaskRunRequest'  # type: str
        self.task_file_path = task_file_path
        self.values_file_path = values_file_path
        self.values = values
        self.timeout = timeout
        self.platform = platform
        self.agent_configuration = agent_configuration
        self.source_location = source_location
        self.credentials = credentials


class FileTaskStep(TaskStepProperties):
    """The properties of a task step.

    Variables are only populated by the server, and will be ignored when sending a request.

    All required parameters must be populated in order to send to Azure.

    :param type: Required. The type of the step.Constant filled by server.  Possible values
     include: "Docker", "FileTask", "EncodedTask".
    :type type: str or ~azure.mgmt.containerregistry.v2019_04_01.models.StepType
    :ivar base_image_dependencies: List of base image dependencies for a step.
    :vartype base_image_dependencies:
     list[~azure.mgmt.containerregistry.v2019_04_01.models.BaseImageDependency]
    :param context_path: The URL(absolute or relative) of the source context for the task step.
    :type context_path: str
    :param context_access_token: The token (git PAT or SAS token of storage account blob)
     associated with the context for a step.
    :type context_access_token: str
    :param task_file_path: Required. The task template/definition file path relative to the source
     context.
    :type task_file_path: str
    :param values_file_path: The task values/parameters file path relative to the source context.
    :type values_file_path: str
    :param values: The collection of overridable values that can be passed when running a task.
    :type values: list[~azure.mgmt.containerregistry.v2019_04_01.models.SetValue]
    """

    _validation = {
        'type': {'required': True},
        'base_image_dependencies': {'readonly': True},
        'task_file_path': {'required': True},
    }

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'base_image_dependencies': {'key': 'baseImageDependencies', 'type': '[BaseImageDependency]'},
        'context_path': {'key': 'contextPath', 'type': 'str'},
        'context_access_token': {'key': 'contextAccessToken', 'type': 'str'},
        'task_file_path': {'key': 'taskFilePath', 'type': 'str'},
        'values_file_path': {'key': 'valuesFilePath', 'type': 'str'},
        'values': {'key': 'values', 'type': '[SetValue]'},
    }

    def __init__(
        self,
        *,
        task_file_path: str,
        context_path: Optional[str] = None,
        context_access_token: Optional[str] = None,
        values_file_path: Optional[str] = None,
        values: Optional[List["SetValue"]] = None,
        **kwargs
    ):
        super(FileTaskStep, self).__init__(context_path=context_path, context_access_token=context_access_token, **kwargs)
        self.type = 'FileTask'  # type: str
        self.task_file_path = task_file_path
        self.values_file_path = values_file_path
        self.values = values


class FileTaskStepUpdateParameters(TaskStepUpdateParameters):
    """The properties of updating a task step.

    All required parameters must be populated in order to send to Azure.

    :param type: Required. The type of the step.Constant filled by server.  Possible values
     include: "Docker", "FileTask", "EncodedTask".
    :type type: str or ~azure.mgmt.containerregistry.v2019_04_01.models.StepType
    :param context_path: The URL(absolute or relative) of the source context for the task step.
    :type context_path: str
    :param context_access_token: The token (git PAT or SAS token of storage account blob)
     associated with the context for a step.
    :type context_access_token: str
    :param task_file_path: The task template/definition file path relative to the source context.
    :type task_file_path: str
    :param values_file_path: The values/parameters file path relative to the source context.
    :type values_file_path: str
    :param values: The collection of overridable values that can be passed when running a task.
    :type values: list[~azure.mgmt.containerregistry.v2019_04_01.models.SetValue]
    """

    _validation = {
        'type': {'required': True},
    }

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'context_path': {'key': 'contextPath', 'type': 'str'},
        'context_access_token': {'key': 'contextAccessToken', 'type': 'str'},
        'task_file_path': {'key': 'taskFilePath', 'type': 'str'},
        'values_file_path': {'key': 'valuesFilePath', 'type': 'str'},
        'values': {'key': 'values', 'type': '[SetValue]'},
    }

    def __init__(
        self,
        *,
        context_path: Optional[str] = None,
        context_access_token: Optional[str] = None,
        task_file_path: Optional[str] = None,
        values_file_path: Optional[str] = None,
        values: Optional[List["SetValue"]] = None,
        **kwargs
    ):
        super(FileTaskStepUpdateParameters, self).__init__(context_path=context_path, context_access_token=context_access_token, **kwargs)
        self.type = 'FileTask'  # type: str
        self.task_file_path = task_file_path
        self.values_file_path = values_file_path
        self.values = values


class IdentityProperties(msrest.serialization.Model):
    """Managed identity for the resource.

    :param principal_id: The principal ID of resource identity.
    :type principal_id: str
    :param tenant_id: The tenant ID of resource.
    :type tenant_id: str
    :param type: The identity type. Possible values include: "SystemAssigned", "UserAssigned",
     "SystemAssigned, UserAssigned", "None".
    :type type: str or ~azure.mgmt.containerregistry.v2019_04_01.models.ResourceIdentityType
    :param user_assigned_identities: The list of user identities associated with the resource. The
     user identity
     dictionary key references will be ARM resource ids in the form:
     '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/
         providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
    :type user_assigned_identities: dict[str,
     ~azure.mgmt.containerregistry.v2019_04_01.models.UserIdentityProperties]
    """

    _attribute_map = {
        'principal_id': {'key': 'principalId', 'type': 'str'},
        'tenant_id': {'key': 'tenantId', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'user_assigned_identities': {'key': 'userAssignedIdentities', 'type': '{UserIdentityProperties}'},
    }

    def __init__(
        self,
        *,
        principal_id: Optional[str] = None,
        tenant_id: Optional[str] = None,
        type: Optional[Union[str, "ResourceIdentityType"]] = None,
        user_assigned_identities: Optional[Dict[str, "UserIdentityProperties"]] = None,
        **kwargs
    ):
        super(IdentityProperties, self).__init__(**kwargs)
        self.principal_id = principal_id
        self.tenant_id = tenant_id
        self.type = type
        self.user_assigned_identities = user_assigned_identities


class ImageDescriptor(msrest.serialization.Model):
    """Properties for a registry image.

    :param registry: The registry login server.
    :type registry: str
    :param repository: The repository name.
    :type repository: str
    :param tag: The tag name.
    :type tag: str
    :param digest: The sha256-based digest of the image manifest.
    :type digest: str
    """

    _attribute_map = {
        'registry': {'key': 'registry', 'type': 'str'},
        'repository': {'key': 'repository', 'type': 'str'},
        'tag': {'key': 'tag', 'type': 'str'},
        'digest': {'key': 'digest', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        registry: Optional[str] = None,
        repository: Optional[str] = None,
        tag: Optional[str] = None,
        digest: Optional[str] = None,
        **kwargs
    ):
        super(ImageDescriptor, self).__init__(**kwargs)
        self.registry = registry
        self.repository = repository
        self.tag = tag
        self.digest = digest


class ImageUpdateTrigger(msrest.serialization.Model):
    """The image update trigger that caused a build.

    :param id: The unique ID of the trigger.
    :type id: str
    :param timestamp: The timestamp when the image update happened.
    :type timestamp: ~datetime.datetime
    :param images: The list of image updates that caused the build.
    :type images: list[~azure.mgmt.containerregistry.v2019_04_01.models.ImageDescriptor]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'timestamp': {'key': 'timestamp', 'type': 'iso-8601'},
        'images': {'key': 'images', 'type': '[ImageDescriptor]'},
    }

    def __init__(
        self,
        *,
        id: Optional[str] = None,
        timestamp: Optional[datetime.datetime] = None,
        images: Optional[List["ImageDescriptor"]] = None,
        **kwargs
    ):
        super(ImageUpdateTrigger, self).__init__(**kwargs)
        self.id = id
        self.timestamp = timestamp
        self.images = images


class PlatformProperties(msrest.serialization.Model):
    """The platform properties against which the run has to happen.

    All required parameters must be populated in order to send to Azure.

    :param os: Required. The operating system type required for the run. Possible values include:
     "Windows", "Linux".
    :type os: str or ~azure.mgmt.containerregistry.v2019_04_01.models.OS
    :param architecture: The OS architecture. Possible values include: "amd64", "x86", "arm".
    :type architecture: str or ~azure.mgmt.containerregistry.v2019_04_01.models.Architecture
    :param variant: Variant of the CPU. Possible values include: "v6", "v7", "v8".
    :type variant: str or ~azure.mgmt.containerregistry.v2019_04_01.models.Variant
    """

    _validation = {
        'os': {'required': True},
    }

    _attribute_map = {
        'os': {'key': 'os', 'type': 'str'},
        'architecture': {'key': 'architecture', 'type': 'str'},
        'variant': {'key': 'variant', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        os: Union[str, "OS"],
        architecture: Optional[Union[str, "Architecture"]] = None,
        variant: Optional[Union[str, "Variant"]] = None,
        **kwargs
    ):
        super(PlatformProperties, self).__init__(**kwargs)
        self.os = os
        self.architecture = architecture
        self.variant = variant


class PlatformUpdateParameters(msrest.serialization.Model):
    """The properties for updating the platform configuration.

    :param os: The operating system type required for the run. Possible values include: "Windows",
     "Linux".
    :type os: str or ~azure.mgmt.containerregistry.v2019_04_01.models.OS
    :param architecture: The OS architecture. Possible values include: "amd64", "x86", "arm".
    :type architecture: str or ~azure.mgmt.containerregistry.v2019_04_01.models.Architecture
    :param variant: Variant of the CPU. Possible values include: "v6", "v7", "v8".
    :type variant: str or ~azure.mgmt.containerregistry.v2019_04_01.models.Variant
    """

    _attribute_map = {
        'os': {'key': 'os', 'type': 'str'},
        'architecture': {'key': 'architecture', 'type': 'str'},
        'variant': {'key': 'variant', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        os: Optional[Union[str, "OS"]] = None,
        architecture: Optional[Union[str, "Architecture"]] = None,
        variant: Optional[Union[str, "Variant"]] = None,
        **kwargs
    ):
        super(PlatformUpdateParameters, self).__init__(**kwargs)
        self.os = os
        self.architecture = architecture
        self.variant = variant


class ProxyResource(msrest.serialization.Model):
    """The resource model definition for a ARM proxy resource. It will have everything other than required location and tags.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar id: The resource ID.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource.
    :vartype type: str
    """

    _validation = {
        'id': {'readonly': True},
        'name': {'readonly': True},
        'type': {'readonly': True},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(ProxyResource, self).__init__(**kwargs)
        self.id = None
        self.name = None
        self.type = None


class Resource(msrest.serialization.Model):
    """An Azure resource.

    Variables are only populated by the server, and will be ignored when sending a request.

    All required parameters must be populated in order to send to Azure.

    :ivar id: The resource ID.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource.
    :vartype type: str
    :param location: Required. The location of the resource. This cannot be changed after the
     resource is created.
    :type location: str
    :param tags: A set of tags. The tags of the resource.
    :type tags: dict[str, str]
    """

    _validation = {
        'id': {'readonly': True},
        'name': {'readonly': True},
        'type': {'readonly': True},
        'location': {'required': True},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'location': {'key': 'location', 'type': 'str'},
        'tags': {'key': 'tags', 'type': '{str}'},
    }

    def __init__(
        self,
        *,
        location: str,
        tags: Optional[Dict[str, str]] = None,
        **kwargs
    ):
        super(Resource, self).__init__(**kwargs)
        self.id = None
        self.name = None
        self.type = None
        self.location = location
        self.tags = tags


class Run(ProxyResource):
    """Run resource properties.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar id: The resource ID.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource.
    :vartype type: str
    :param run_id: The unique identifier for the run.
    :type run_id: str
    :param status: The current status of the run. Possible values include: "Queued", "Started",
     "Running", "Succeeded", "Failed", "Canceled", "Error", "Timeout".
    :type status: str or ~azure.mgmt.containerregistry.v2019_04_01.models.RunStatus
    :param last_updated_time: The last updated time for the run.
    :type last_updated_time: ~datetime.datetime
    :param run_type: The type of run. Possible values include: "QuickBuild", "QuickRun",
     "AutoBuild", "AutoRun".
    :type run_type: str or ~azure.mgmt.containerregistry.v2019_04_01.models.RunType
    :param create_time: The time the run was scheduled.
    :type create_time: ~datetime.datetime
    :param start_time: The time the run started.
    :type start_time: ~datetime.datetime
    :param finish_time: The time the run finished.
    :type finish_time: ~datetime.datetime
    :param output_images: The list of all images that were generated from the run. This is
     applicable if the run generates base image dependencies.
    :type output_images: list[~azure.mgmt.containerregistry.v2019_04_01.models.ImageDescriptor]
    :param task: The task against which run was scheduled.
    :type task: str
    :param image_update_trigger: The image update trigger that caused the run. This is applicable
     if the task has base image trigger configured.
    :type image_update_trigger: ~azure.mgmt.containerregistry.v2019_04_01.models.ImageUpdateTrigger
    :param source_trigger: The source trigger that caused the run.
    :type source_trigger: ~azure.mgmt.containerregistry.v2019_04_01.models.SourceTriggerDescriptor
    :param platform: The platform properties against which the run will happen.
    :type platform: ~azure.mgmt.containerregistry.v2019_04_01.models.PlatformProperties
    :param agent_configuration: The machine configuration of the run agent.
    :type agent_configuration: ~azure.mgmt.containerregistry.v2019_04_01.models.AgentProperties
    :param source_registry_auth: The scope of the credentials that were used to login to the source
     registry during this run.
    :type source_registry_auth: str
    :param custom_registries: The list of custom registries that were logged in during this run.
    :type custom_registries: list[str]
    :ivar run_error_message: The error message received from backend systems after the run is
     scheduled.
    :vartype run_error_message: str
    :param provisioning_state: The provisioning state of a run. Possible values include:
     "Creating", "Updating", "Deleting", "Succeeded", "Failed", "Canceled".
    :type provisioning_state: str or
     ~azure.mgmt.containerregistry.v2019_04_01.models.ProvisioningState
    :param is_archive_enabled: The value that indicates whether archiving is enabled or not.
    :type is_archive_enabled: bool
    :param timer_trigger: The timer trigger that caused the run.
    :type timer_trigger: ~azure.mgmt.containerregistry.v2019_04_01.models.TimerTriggerDescriptor
    """

    _validation = {
        'id': {'readonly': True},
        'name': {'readonly': True},
        'type': {'readonly': True},
        'run_error_message': {'readonly': True},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'run_id': {'key': 'properties.runId', 'type': 'str'},
        'status': {'key': 'properties.status', 'type': 'str'},
        'last_updated_time': {'key': 'properties.lastUpdatedTime', 'type': 'iso-8601'},
        'run_type': {'key': 'properties.runType', 'type': 'str'},
        'create_time': {'key': 'properties.createTime', 'type': 'iso-8601'},
        'start_time': {'key': 'properties.startTime', 'type': 'iso-8601'},
        'finish_time': {'key': 'properties.finishTime', 'type': 'iso-8601'},
        'output_images': {'key': 'properties.outputImages', 'type': '[ImageDescriptor]'},
        'task': {'key': 'properties.task', 'type': 'str'},
        'image_update_trigger': {'key': 'properties.imageUpdateTrigger', 'type': 'ImageUpdateTrigger'},
        'source_trigger': {'key': 'properties.sourceTrigger', 'type': 'SourceTriggerDescriptor'},
        'platform': {'key': 'properties.platform', 'type': 'PlatformProperties'},
        'agent_configuration': {'key': 'properties.agentConfiguration', 'type': 'AgentProperties'},
        'source_registry_auth': {'key': 'properties.sourceRegistryAuth', 'type': 'str'},
        'custom_registries': {'key': 'properties.customRegistries', 'type': '[str]'},
        'run_error_message': {'key': 'properties.runErrorMessage', 'type': 'str'},
        'provisioning_state': {'key': 'properties.provisioningState', 'type': 'str'},
        'is_archive_enabled': {'key': 'properties.isArchiveEnabled', 'type': 'bool'},
        'timer_trigger': {'key': 'properties.timerTrigger', 'type': 'TimerTriggerDescriptor'},
    }

    def __init__(
        self,
        *,
        run_id: Optional[str] = None,
        status: Optional[Union[str, "RunStatus"]] = None,
        last_updated_time: Optional[datetime.datetime] = None,
        run_type: Optional[Union[str, "RunType"]] = None,
        create_time: Optional[datetime.datetime] = None,
        start_time: Optional[datetime.datetime] = None,
        finish_time: Optional[datetime.datetime] = None,
        output_images: Optional[List["ImageDescriptor"]] = None,
        task: Optional[str] = None,
        image_update_trigger: Optional["ImageUpdateTrigger"] = None,
        source_trigger: Optional["SourceTriggerDescriptor"] = None,
        platform: Optional["PlatformProperties"] = None,
        agent_configuration: Optional["AgentProperties"] = None,
        source_registry_auth: Optional[str] = None,
        custom_registries: Optional[List[str]] = None,
        provisioning_state: Optional[Union[str, "ProvisioningState"]] = None,
        is_archive_enabled: Optional[bool] = False,
        timer_trigger: Optional["TimerTriggerDescriptor"] = None,
        **kwargs
    ):
        super(Run, self).__init__(**kwargs)
        self.run_id = run_id
        self.status = status
        self.last_updated_time = last_updated_time
        self.run_type = run_type
        self.create_time = create_time
        self.start_time = start_time
        self.finish_time = finish_time
        self.output_images = output_images
        self.task = task
        self.image_update_trigger = image_update_trigger
        self.source_trigger = source_trigger
        self.platform = platform
        self.agent_configuration = agent_configuration
        self.source_registry_auth = source_registry_auth
        self.custom_registries = custom_registries
        self.run_error_message = None
        self.provisioning_state = provisioning_state
        self.is_archive_enabled = is_archive_enabled
        self.timer_trigger = timer_trigger


class RunFilter(msrest.serialization.Model):
    """Properties that are enabled for Odata querying on runs.

    :param run_id: The unique identifier for the run.
    :type run_id: str
    :param run_type: The type of run. Possible values include: "QuickBuild", "QuickRun",
     "AutoBuild", "AutoRun".
    :type run_type: str or ~azure.mgmt.containerregistry.v2019_04_01.models.RunType
    :param status: The current status of the run. Possible values include: "Queued", "Started",
     "Running", "Succeeded", "Failed", "Canceled", "Error", "Timeout".
    :type status: str or ~azure.mgmt.containerregistry.v2019_04_01.models.RunStatus
    :param create_time: The create time for a run.
    :type create_time: ~datetime.datetime
    :param finish_time: The time the run finished.
    :type finish_time: ~datetime.datetime
    :param output_image_manifests: The list of comma-separated image manifests that were generated
     from the run. This is applicable if the run is of
     build type.
    :type output_image_manifests: str
    :param is_archive_enabled: The value that indicates whether archiving is enabled or not.
    :type is_archive_enabled: bool
    :param task_name: The name of the task that the run corresponds to.
    :type task_name: str
    """

    _attribute_map = {
        'run_id': {'key': 'runId', 'type': 'str'},
        'run_type': {'key': 'runType', 'type': 'str'},
        'status': {'key': 'status', 'type': 'str'},
        'create_time': {'key': 'createTime', 'type': 'iso-8601'},
        'finish_time': {'key': 'finishTime', 'type': 'iso-8601'},
        'output_image_manifests': {'key': 'outputImageManifests', 'type': 'str'},
        'is_archive_enabled': {'key': 'isArchiveEnabled', 'type': 'bool'},
        'task_name': {'key': 'taskName', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        run_id: Optional[str] = None,
        run_type: Optional[Union[str, "RunType"]] = None,
        status: Optional[Union[str, "RunStatus"]] = None,
        create_time: Optional[datetime.datetime] = None,
        finish_time: Optional[datetime.datetime] = None,
        output_image_manifests: Optional[str] = None,
        is_archive_enabled: Optional[bool] = None,
        task_name: Optional[str] = None,
        **kwargs
    ):
        super(RunFilter, self).__init__(**kwargs)
        self.run_id = run_id
        self.run_type = run_type
        self.status = status
        self.create_time = create_time
        self.finish_time = finish_time
        self.output_image_manifests = output_image_manifests
        self.is_archive_enabled = is_archive_enabled
        self.task_name = task_name


class RunGetLogResult(msrest.serialization.Model):
    """The result of get log link operation.

    :param log_link: The link to logs for a run on a azure container registry.
    :type log_link: str
    """

    _attribute_map = {
        'log_link': {'key': 'logLink', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        log_link: Optional[str] = None,
        **kwargs
    ):
        super(RunGetLogResult, self).__init__(**kwargs)
        self.log_link = log_link


class RunListResult(msrest.serialization.Model):
    """Collection of runs.

    :param value: The collection value.
    :type value: list[~azure.mgmt.containerregistry.v2019_04_01.models.Run]
    :param next_link: The URI that can be used to request the next set of paged results.
    :type next_link: str
    """

    _attribute_map = {
        'value': {'key': 'value', 'type': '[Run]'},
        'next_link': {'key': 'nextLink', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        value: Optional[List["Run"]] = None,
        next_link: Optional[str] = None,
        **kwargs
    ):
        super(RunListResult, self).__init__(**kwargs)
        self.value = value
        self.next_link = next_link


class RunUpdateParameters(msrest.serialization.Model):
    """The set of run properties that can be updated.

    :param is_archive_enabled: The value that indicates whether archiving is enabled or not.
    :type is_archive_enabled: bool
    """

    _attribute_map = {
        'is_archive_enabled': {'key': 'isArchiveEnabled', 'type': 'bool'},
    }

    def __init__(
        self,
        *,
        is_archive_enabled: Optional[bool] = None,
        **kwargs
    ):
        super(RunUpdateParameters, self).__init__(**kwargs)
        self.is_archive_enabled = is_archive_enabled


class SecretObject(msrest.serialization.Model):
    """Describes the properties of a secret object value.

    :param value: The value of the secret. The format of this value will be determined
     based on the type of the secret object. If the type is Opaque, the value will be
     used as is without any modification.
    :type value: str
    :param type: The type of the secret object which determines how the value of the secret object
     has to be
     interpreted. Possible values include: "Opaque", "Vaultsecret".
    :type type: str or ~azure.mgmt.containerregistry.v2019_04_01.models.SecretObjectType
    """

    _attribute_map = {
        'value': {'key': 'value', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        value: Optional[str] = None,
        type: Optional[Union[str, "SecretObjectType"]] = None,
        **kwargs
    ):
        super(SecretObject, self).__init__(**kwargs)
        self.value = value
        self.type = type


class SetValue(msrest.serialization.Model):
    """The properties of a overridable value that can be passed to a task template.

    All required parameters must be populated in order to send to Azure.

    :param name: Required. The name of the overridable value.
    :type name: str
    :param value: Required. The overridable value.
    :type value: str
    :param is_secret: Flag to indicate whether the value represents a secret or not.
    :type is_secret: bool
    """

    _validation = {
        'name': {'required': True},
        'value': {'required': True},
    }

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'value': {'key': 'value', 'type': 'str'},
        'is_secret': {'key': 'isSecret', 'type': 'bool'},
    }

    def __init__(
        self,
        *,
        name: str,
        value: str,
        is_secret: Optional[bool] = False,
        **kwargs
    ):
        super(SetValue, self).__init__(**kwargs)
        self.name = name
        self.value = value
        self.is_secret = is_secret


class SourceProperties(msrest.serialization.Model):
    """The properties of the source code repository.

    All required parameters must be populated in order to send to Azure.

    :param source_control_type: Required. The type of source control service. Possible values
     include: "Github", "VisualStudioTeamService".
    :type source_control_type: str or
     ~azure.mgmt.containerregistry.v2019_04_01.models.SourceControlType
    :param repository_url: Required. The full URL to the source code repository.
    :type repository_url: str
    :param branch: The branch name of the source code.
    :type branch: str
    :param source_control_auth_properties: The authorization properties for accessing the source
     code repository and to set up
     webhooks for notifications.
    :type source_control_auth_properties: ~azure.mgmt.containerregistry.v2019_04_01.models.AuthInfo
    """

    _validation = {
        'source_control_type': {'required': True},
        'repository_url': {'required': True},
    }

    _attribute_map = {
        'source_control_type': {'key': 'sourceControlType', 'type': 'str'},
        'repository_url': {'key': 'repositoryUrl', 'type': 'str'},
        'branch': {'key': 'branch', 'type': 'str'},
        'source_control_auth_properties': {'key': 'sourceControlAuthProperties', 'type': 'AuthInfo'},
    }

    def __init__(
        self,
        *,
        source_control_type: Union[str, "SourceControlType"],
        repository_url: str,
        branch: Optional[str] = None,
        source_control_auth_properties: Optional["AuthInfo"] = None,
        **kwargs
    ):
        super(SourceProperties, self).__init__(**kwargs)
        self.source_control_type = source_control_type
        self.repository_url = repository_url
        self.branch = branch
        self.source_control_auth_properties = source_control_auth_properties


class SourceRegistryCredentials(msrest.serialization.Model):
    """Describes the credential parameters for accessing the source registry.

    :param login_mode: The authentication mode which determines the source registry login scope.
     The credentials for the source registry
     will be generated using the given scope. These credentials will be used to login to
     the source registry during the run. Possible values include: "None", "Default".
    :type login_mode: str or
     ~azure.mgmt.containerregistry.v2019_04_01.models.SourceRegistryLoginMode
    """

    _attribute_map = {
        'login_mode': {'key': 'loginMode', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        login_mode: Optional[Union[str, "SourceRegistryLoginMode"]] = None,
        **kwargs
    ):
        super(SourceRegistryCredentials, self).__init__(**kwargs)
        self.login_mode = login_mode


class SourceTrigger(msrest.serialization.Model):
    """The properties of a source based trigger.

    All required parameters must be populated in order to send to Azure.

    :param source_repository: Required. The properties that describes the source(code) for the
     task.
    :type source_repository: ~azure.mgmt.containerregistry.v2019_04_01.models.SourceProperties
    :param source_trigger_events: Required. The source event corresponding to the trigger.
    :type source_trigger_events: list[str or
     ~azure.mgmt.containerregistry.v2019_04_01.models.SourceTriggerEvent]
    :param status: The current status of trigger. Possible values include: "Disabled", "Enabled".
    :type status: str or ~azure.mgmt.containerregistry.v2019_04_01.models.TriggerStatus
    :param name: Required. The name of the trigger.
    :type name: str
    """

    _validation = {
        'source_repository': {'required': True},
        'source_trigger_events': {'required': True},
        'name': {'required': True},
    }

    _attribute_map = {
        'source_repository': {'key': 'sourceRepository', 'type': 'SourceProperties'},
        'source_trigger_events': {'key': 'sourceTriggerEvents', 'type': '[str]'},
        'status': {'key': 'status', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        source_repository: "SourceProperties",
        source_trigger_events: List[Union[str, "SourceTriggerEvent"]],
        name: str,
        status: Optional[Union[str, "TriggerStatus"]] = None,
        **kwargs
    ):
        super(SourceTrigger, self).__init__(**kwargs)
        self.source_repository = source_repository
        self.source_trigger_events = source_trigger_events
        self.status = status
        self.name = name


class SourceTriggerDescriptor(msrest.serialization.Model):
    """The source trigger that caused a run.

    :param id: The unique ID of the trigger.
    :type id: str
    :param event_type: The event type of the trigger.
    :type event_type: str
    :param commit_id: The unique ID that identifies a commit.
    :type commit_id: str
    :param pull_request_id: The unique ID that identifies pull request.
    :type pull_request_id: str
    :param repository_url: The repository URL.
    :type repository_url: str
    :param branch_name: The branch name in the repository.
    :type branch_name: str
    :param provider_type: The source control provider type.
    :type provider_type: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'event_type': {'key': 'eventType', 'type': 'str'},
        'commit_id': {'key': 'commitId', 'type': 'str'},
        'pull_request_id': {'key': 'pullRequestId', 'type': 'str'},
        'repository_url': {'key': 'repositoryUrl', 'type': 'str'},
        'branch_name': {'key': 'branchName', 'type': 'str'},
        'provider_type': {'key': 'providerType', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        id: Optional[str] = None,
        event_type: Optional[str] = None,
        commit_id: Optional[str] = None,
        pull_request_id: Optional[str] = None,
        repository_url: Optional[str] = None,
        branch_name: Optional[str] = None,
        provider_type: Optional[str] = None,
        **kwargs
    ):
        super(SourceTriggerDescriptor, self).__init__(**kwargs)
        self.id = id
        self.event_type = event_type
        self.commit_id = commit_id
        self.pull_request_id = pull_request_id
        self.repository_url = repository_url
        self.branch_name = branch_name
        self.provider_type = provider_type


class SourceTriggerUpdateParameters(msrest.serialization.Model):
    """The properties for updating a source based trigger.

    All required parameters must be populated in order to send to Azure.

    :param source_repository: The properties that describes the source(code) for the task.
    :type source_repository:
     ~azure.mgmt.containerregistry.v2019_04_01.models.SourceUpdateParameters
    :param source_trigger_events: The source event corresponding to the trigger.
    :type source_trigger_events: list[str or
     ~azure.mgmt.containerregistry.v2019_04_01.models.SourceTriggerEvent]
    :param status: The current status of trigger. Possible values include: "Disabled", "Enabled".
    :type status: str or ~azure.mgmt.containerregistry.v2019_04_01.models.TriggerStatus
    :param name: Required. The name of the trigger.
    :type name: str
    """

    _validation = {
        'name': {'required': True},
    }

    _attribute_map = {
        'source_repository': {'key': 'sourceRepository', 'type': 'SourceUpdateParameters'},
        'source_trigger_events': {'key': 'sourceTriggerEvents', 'type': '[str]'},
        'status': {'key': 'status', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        name: str,
        source_repository: Optional["SourceUpdateParameters"] = None,
        source_trigger_events: Optional[List[Union[str, "SourceTriggerEvent"]]] = None,
        status: Optional[Union[str, "TriggerStatus"]] = None,
        **kwargs
    ):
        super(SourceTriggerUpdateParameters, self).__init__(**kwargs)
        self.source_repository = source_repository
        self.source_trigger_events = source_trigger_events
        self.status = status
        self.name = name


class SourceUpdateParameters(msrest.serialization.Model):
    """The properties for updating the source code repository.

    :param source_control_type: The type of source control service. Possible values include:
     "Github", "VisualStudioTeamService".
    :type source_control_type: str or
     ~azure.mgmt.containerregistry.v2019_04_01.models.SourceControlType
    :param repository_url: The full URL to the source code repository.
    :type repository_url: str
    :param branch: The branch name of the source code.
    :type branch: str
    :param source_control_auth_properties: The authorization properties for accessing the source
     code repository and to set up
     webhooks for notifications.
    :type source_control_auth_properties:
     ~azure.mgmt.containerregistry.v2019_04_01.models.AuthInfoUpdateParameters
    """

    _attribute_map = {
        'source_control_type': {'key': 'sourceControlType', 'type': 'str'},
        'repository_url': {'key': 'repositoryUrl', 'type': 'str'},
        'branch': {'key': 'branch', 'type': 'str'},
        'source_control_auth_properties': {'key': 'sourceControlAuthProperties', 'type': 'AuthInfoUpdateParameters'},
    }

    def __init__(
        self,
        *,
        source_control_type: Optional[Union[str, "SourceControlType"]] = None,
        repository_url: Optional[str] = None,
        branch: Optional[str] = None,
        source_control_auth_properties: Optional["AuthInfoUpdateParameters"] = None,
        **kwargs
    ):
        super(SourceUpdateParameters, self).__init__(**kwargs)
        self.source_control_type = source_control_type
        self.repository_url = repository_url
        self.branch = branch
        self.source_control_auth_properties = source_control_auth_properties


class SourceUploadDefinition(msrest.serialization.Model):
    """The properties of a response to source upload request.

    :param upload_url: The URL where the client can upload the source.
    :type upload_url: str
    :param relative_path: The relative path to the source. This is used to submit the subsequent
     queue build request.
    :type relative_path: str
    """

    _attribute_map = {
        'upload_url': {'key': 'uploadUrl', 'type': 'str'},
        'relative_path': {'key': 'relativePath', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        upload_url: Optional[str] = None,
        relative_path: Optional[str] = None,
        **kwargs
    ):
        super(SourceUploadDefinition, self).__init__(**kwargs)
        self.upload_url = upload_url
        self.relative_path = relative_path


class Task(Resource):
    """The task that has the ARM resource and task properties. 
The task will have all information to schedule a run against it.

    Variables are only populated by the server, and will be ignored when sending a request.

    All required parameters must be populated in order to send to Azure.

    :ivar id: The resource ID.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource.
    :vartype type: str
    :param location: Required. The location of the resource. This cannot be changed after the
     resource is created.
    :type location: str
    :param tags: A set of tags. The tags of the resource.
    :type tags: dict[str, str]
    :param identity: Identity for the resource.
    :type identity: ~azure.mgmt.containerregistry.v2019_04_01.models.IdentityProperties
    :ivar provisioning_state: The provisioning state of the task. Possible values include:
     "Creating", "Updating", "Deleting", "Succeeded", "Failed", "Canceled".
    :vartype provisioning_state: str or
     ~azure.mgmt.containerregistry.v2019_04_01.models.ProvisioningState
    :ivar creation_date: The creation date of task.
    :vartype creation_date: ~datetime.datetime
    :param status: The current status of task. Possible values include: "Disabled", "Enabled".
    :type status: str or ~azure.mgmt.containerregistry.v2019_04_01.models.TaskStatus
    :param platform: The platform properties against which the run has to happen.
    :type platform: ~azure.mgmt.containerregistry.v2019_04_01.models.PlatformProperties
    :param agent_configuration: The machine configuration of the run agent.
    :type agent_configuration: ~azure.mgmt.containerregistry.v2019_04_01.models.AgentProperties
    :param timeout: Run timeout in seconds.
    :type timeout: int
    :param step: The properties of a task step.
    :type step: ~azure.mgmt.containerregistry.v2019_04_01.models.TaskStepProperties
    :param trigger: The properties that describe all triggers for the task.
    :type trigger: ~azure.mgmt.containerregistry.v2019_04_01.models.TriggerProperties
    :param credentials: The properties that describes a set of credentials that will be used when
     this run is invoked.
    :type credentials: ~azure.mgmt.containerregistry.v2019_04_01.models.Credentials
    """

    _validation = {
        'id': {'readonly': True},
        'name': {'readonly': True},
        'type': {'readonly': True},
        'location': {'required': True},
        'provisioning_state': {'readonly': True},
        'creation_date': {'readonly': True},
        'timeout': {'maximum': 28800, 'minimum': 300},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'location': {'key': 'location', 'type': 'str'},
        'tags': {'key': 'tags', 'type': '{str}'},
        'identity': {'key': 'identity', 'type': 'IdentityProperties'},
        'provisioning_state': {'key': 'properties.provisioningState', 'type': 'str'},
        'creation_date': {'key': 'properties.creationDate', 'type': 'iso-8601'},
        'status': {'key': 'properties.status', 'type': 'str'},
        'platform': {'key': 'properties.platform', 'type': 'PlatformProperties'},
        'agent_configuration': {'key': 'properties.agentConfiguration', 'type': 'AgentProperties'},
        'timeout': {'key': 'properties.timeout', 'type': 'int'},
        'step': {'key': 'properties.step', 'type': 'TaskStepProperties'},
        'trigger': {'key': 'properties.trigger', 'type': 'TriggerProperties'},
        'credentials': {'key': 'properties.credentials', 'type': 'Credentials'},
    }

    def __init__(
        self,
        *,
        location: str,
        tags: Optional[Dict[str, str]] = None,
        identity: Optional["IdentityProperties"] = None,
        status: Optional[Union[str, "TaskStatus"]] = None,
        platform: Optional["PlatformProperties"] = None,
        agent_configuration: Optional["AgentProperties"] = None,
        timeout: Optional[int] = 3600,
        step: Optional["TaskStepProperties"] = None,
        trigger: Optional["TriggerProperties"] = None,
        credentials: Optional["Credentials"] = None,
        **kwargs
    ):
        super(Task, self).__init__(location=location, tags=tags, **kwargs)
        self.identity = identity
        self.provisioning_state = None
        self.creation_date = None
        self.status = status
        self.platform = platform
        self.agent_configuration = agent_configuration
        self.timeout = timeout
        self.step = step
        self.trigger = trigger
        self.credentials = credentials


class TaskListResult(msrest.serialization.Model):
    """The collection of tasks.

    :param value: The collection value.
    :type value: list[~azure.mgmt.containerregistry.v2019_04_01.models.Task]
    :param next_link: The URI that can be used to request the next set of paged results.
    :type next_link: str
    """

    _attribute_map = {
        'value': {'key': 'value', 'type': '[Task]'},
        'next_link': {'key': 'nextLink', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        value: Optional[List["Task"]] = None,
        next_link: Optional[str] = None,
        **kwargs
    ):
        super(TaskListResult, self).__init__(**kwargs)
        self.value = value
        self.next_link = next_link


class TaskRunRequest(RunRequest):
    """The parameters for a task run request.

    All required parameters must be populated in order to send to Azure.

    :param type: Required. The type of the run request.Constant filled by server.
    :type type: str
    :param is_archive_enabled: The value that indicates whether archiving is enabled for the run or
     not.
    :type is_archive_enabled: bool
    :param task_name: Required. The name of task against which run has to be queued.
    :type task_name: str
    :param values: The collection of overridable values that can be passed when running a task.
    :type values: list[~azure.mgmt.containerregistry.v2019_04_01.models.SetValue]
    """

    _validation = {
        'type': {'required': True},
        'task_name': {'required': True},
    }

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'is_archive_enabled': {'key': 'isArchiveEnabled', 'type': 'bool'},
        'task_name': {'key': 'taskName', 'type': 'str'},
        'values': {'key': 'values', 'type': '[SetValue]'},
    }

    def __init__(
        self,
        *,
        task_name: str,
        is_archive_enabled: Optional[bool] = False,
        values: Optional[List["SetValue"]] = None,
        **kwargs
    ):
        super(TaskRunRequest, self).__init__(is_archive_enabled=is_archive_enabled, **kwargs)
        self.type = 'TaskRunRequest'  # type: str
        self.task_name = task_name
        self.values = values


class TaskUpdateParameters(msrest.serialization.Model):
    """The parameters for updating a task.

    :param identity: Identity for the resource.
    :type identity: ~azure.mgmt.containerregistry.v2019_04_01.models.IdentityProperties
    :param tags: A set of tags. The ARM resource tags.
    :type tags: dict[str, str]
    :param status: The current status of task. Possible values include: "Disabled", "Enabled".
    :type status: str or ~azure.mgmt.containerregistry.v2019_04_01.models.TaskStatus
    :param platform: The platform properties against which the run has to happen.
    :type platform: ~azure.mgmt.containerregistry.v2019_04_01.models.PlatformUpdateParameters
    :param agent_configuration: The machine configuration of the run agent.
    :type agent_configuration: ~azure.mgmt.containerregistry.v2019_04_01.models.AgentProperties
    :param timeout: Run timeout in seconds.
    :type timeout: int
    :param step: The properties for updating a task step.
    :type step: ~azure.mgmt.containerregistry.v2019_04_01.models.TaskStepUpdateParameters
    :param trigger: The properties for updating trigger properties.
    :type trigger: ~azure.mgmt.containerregistry.v2019_04_01.models.TriggerUpdateParameters
    :param credentials: The parameters that describes a set of credentials that will be used when
     this run is invoked.
    :type credentials: ~azure.mgmt.containerregistry.v2019_04_01.models.Credentials
    """

    _attribute_map = {
        'identity': {'key': 'identity', 'type': 'IdentityProperties'},
        'tags': {'key': 'tags', 'type': '{str}'},
        'status': {'key': 'properties.status', 'type': 'str'},
        'platform': {'key': 'properties.platform', 'type': 'PlatformUpdateParameters'},
        'agent_configuration': {'key': 'properties.agentConfiguration', 'type': 'AgentProperties'},
        'timeout': {'key': 'properties.timeout', 'type': 'int'},
        'step': {'key': 'properties.step', 'type': 'TaskStepUpdateParameters'},
        'trigger': {'key': 'properties.trigger', 'type': 'TriggerUpdateParameters'},
        'credentials': {'key': 'properties.credentials', 'type': 'Credentials'},
    }

    def __init__(
        self,
        *,
        identity: Optional["IdentityProperties"] = None,
        tags: Optional[Dict[str, str]] = None,
        status: Optional[Union[str, "TaskStatus"]] = None,
        platform: Optional["PlatformUpdateParameters"] = None,
        agent_configuration: Optional["AgentProperties"] = None,
        timeout: Optional[int] = None,
        step: Optional["TaskStepUpdateParameters"] = None,
        trigger: Optional["TriggerUpdateParameters"] = None,
        credentials: Optional["Credentials"] = None,
        **kwargs
    ):
        super(TaskUpdateParameters, self).__init__(**kwargs)
        self.identity = identity
        self.tags = tags
        self.status = status
        self.platform = platform
        self.agent_configuration = agent_configuration
        self.timeout = timeout
        self.step = step
        self.trigger = trigger
        self.credentials = credentials


class TimerTrigger(msrest.serialization.Model):
    """The properties of a timer trigger.

    All required parameters must be populated in order to send to Azure.

    :param schedule: Required. The CRON expression for the task schedule.
    :type schedule: str
    :param status: The current status of trigger. Possible values include: "Disabled", "Enabled".
    :type status: str or ~azure.mgmt.containerregistry.v2019_04_01.models.TriggerStatus
    :param name: Required. The name of the trigger.
    :type name: str
    """

    _validation = {
        'schedule': {'required': True},
        'name': {'required': True},
    }

    _attribute_map = {
        'schedule': {'key': 'schedule', 'type': 'str'},
        'status': {'key': 'status', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        schedule: str,
        name: str,
        status: Optional[Union[str, "TriggerStatus"]] = None,
        **kwargs
    ):
        super(TimerTrigger, self).__init__(**kwargs)
        self.schedule = schedule
        self.status = status
        self.name = name


class TimerTriggerDescriptor(msrest.serialization.Model):
    """TimerTriggerDescriptor.

    :param timer_trigger_name: The timer trigger name that caused the run.
    :type timer_trigger_name: str
    :param schedule_occurrence: The occurrence that triggered the run.
    :type schedule_occurrence: str
    """

    _attribute_map = {
        'timer_trigger_name': {'key': 'timerTriggerName', 'type': 'str'},
        'schedule_occurrence': {'key': 'scheduleOccurrence', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        timer_trigger_name: Optional[str] = None,
        schedule_occurrence: Optional[str] = None,
        **kwargs
    ):
        super(TimerTriggerDescriptor, self).__init__(**kwargs)
        self.timer_trigger_name = timer_trigger_name
        self.schedule_occurrence = schedule_occurrence


class TimerTriggerUpdateParameters(msrest.serialization.Model):
    """The properties for updating a timer trigger.

    All required parameters must be populated in order to send to Azure.

    :param schedule: The CRON expression for the task schedule.
    :type schedule: str
    :param status: The current status of trigger. Possible values include: "Disabled", "Enabled".
    :type status: str or ~azure.mgmt.containerregistry.v2019_04_01.models.TriggerStatus
    :param name: Required. The name of the trigger.
    :type name: str
    """

    _validation = {
        'name': {'required': True},
    }

    _attribute_map = {
        'schedule': {'key': 'schedule', 'type': 'str'},
        'status': {'key': 'status', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        name: str,
        schedule: Optional[str] = None,
        status: Optional[Union[str, "TriggerStatus"]] = None,
        **kwargs
    ):
        super(TimerTriggerUpdateParameters, self).__init__(**kwargs)
        self.schedule = schedule
        self.status = status
        self.name = name


class TriggerProperties(msrest.serialization.Model):
    """The properties of a trigger.

    :param timer_triggers: The collection of timer triggers.
    :type timer_triggers: list[~azure.mgmt.containerregistry.v2019_04_01.models.TimerTrigger]
    :param source_triggers: The collection of triggers based on source code repository.
    :type source_triggers: list[~azure.mgmt.containerregistry.v2019_04_01.models.SourceTrigger]
    :param base_image_trigger: The trigger based on base image dependencies.
    :type base_image_trigger: ~azure.mgmt.containerregistry.v2019_04_01.models.BaseImageTrigger
    """

    _attribute_map = {
        'timer_triggers': {'key': 'timerTriggers', 'type': '[TimerTrigger]'},
        'source_triggers': {'key': 'sourceTriggers', 'type': '[SourceTrigger]'},
        'base_image_trigger': {'key': 'baseImageTrigger', 'type': 'BaseImageTrigger'},
    }

    def __init__(
        self,
        *,
        timer_triggers: Optional[List["TimerTrigger"]] = None,
        source_triggers: Optional[List["SourceTrigger"]] = None,
        base_image_trigger: Optional["BaseImageTrigger"] = None,
        **kwargs
    ):
        super(TriggerProperties, self).__init__(**kwargs)
        self.timer_triggers = timer_triggers
        self.source_triggers = source_triggers
        self.base_image_trigger = base_image_trigger


class TriggerUpdateParameters(msrest.serialization.Model):
    """The properties for updating triggers.

    :param timer_triggers: The collection of timer triggers.
    :type timer_triggers:
     list[~azure.mgmt.containerregistry.v2019_04_01.models.TimerTriggerUpdateParameters]
    :param source_triggers: The collection of triggers based on source code repository.
    :type source_triggers:
     list[~azure.mgmt.containerregistry.v2019_04_01.models.SourceTriggerUpdateParameters]
    :param base_image_trigger: The trigger based on base image dependencies.
    :type base_image_trigger:
     ~azure.mgmt.containerregistry.v2019_04_01.models.BaseImageTriggerUpdateParameters
    """

    _attribute_map = {
        'timer_triggers': {'key': 'timerTriggers', 'type': '[TimerTriggerUpdateParameters]'},
        'source_triggers': {'key': 'sourceTriggers', 'type': '[SourceTriggerUpdateParameters]'},
        'base_image_trigger': {'key': 'baseImageTrigger', 'type': 'BaseImageTriggerUpdateParameters'},
    }

    def __init__(
        self,
        *,
        timer_triggers: Optional[List["TimerTriggerUpdateParameters"]] = None,
        source_triggers: Optional[List["SourceTriggerUpdateParameters"]] = None,
        base_image_trigger: Optional["BaseImageTriggerUpdateParameters"] = None,
        **kwargs
    ):
        super(TriggerUpdateParameters, self).__init__(**kwargs)
        self.timer_triggers = timer_triggers
        self.source_triggers = source_triggers
        self.base_image_trigger = base_image_trigger


class UserIdentityProperties(msrest.serialization.Model):
    """UserIdentityProperties.

    :param principal_id: The principal id of user assigned identity.
    :type principal_id: str
    :param client_id: The client id of user assigned identity.
    :type client_id: str
    """

    _attribute_map = {
        'principal_id': {'key': 'principalId', 'type': 'str'},
        'client_id': {'key': 'clientId', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        principal_id: Optional[str] = None,
        client_id: Optional[str] = None,
        **kwargs
    ):
        super(UserIdentityProperties, self).__init__(**kwargs)
        self.principal_id = principal_id
        self.client_id = client_id
